//
// "$Id DockRegion.cxx Thu Feb 5 2006 z_hossain $"
//
// Dockability for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2005 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to: 
//
//             md_z_hossain@yahoo.com
//

#include <fltk/Window.h>
#include <fltk/draw.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>

#include "DockRegion.h"

#define ABS(a) (((a) < 0) ? (-(a)) : (a))
#define TOLERANCE 5

using namespace fltk;

dgDockHolder::dgDockHolder(int x, int y, int w, int h, Flags a) : Rectangle(x, y, w, h) {
	children_ = 0;
	align_holder(a);
}

dgDockHolder::~dgDockHolder() {
	delete [] array_;
}

void dgDockHolder::resize_range() {
	switch(align_) {
		case ALIGN_TOP:
			range(x() - 10, y() - 10, w() + 20, h() + 40);
			break;
		case ALIGN_BOTTOM:
			range(x() - 10, y() - 60, w() + 10, h() + 70);
			break;
		case ALIGN_LEFT:
			range(x() - 10, y() - 10, w() + 60, h() + 15);
			break;
		case ALIGN_RIGHT:
			range(x() - 50, y() - 5, w() + 55, h() + 10);
			break;
	}
}

void dgDockHolder::add(DockableGroup *o) {
	if (children_ == 0) {
		array_ = new DockableGroup*[1];
		array_[0] = o;
	} else {
		if (!(children_ & (children_-1))) {
			DockableGroup** newarray = new DockableGroup*[2*children_];
			memcpy(newarray, array_, children_*sizeof(DockableGroup*));
			delete[] array_;
			array_ = newarray;
		}
		array_[children_] = o;
	}
	++children_;
}

void dgDockHolder::remove(DockableGroup *o) {
	int k = have(o);
	if(k == -1) return;
	children_--;
	for (int i = k; i < children_; ++i) array_[i] = array_[i+1];
}

int dgDockHolder::have(DockableGroup *o) {
	DockableGroup*const* a = array_;
	for (int i = children_; i--;) {
		if((*a++) == o)
			return (children_ - 1 - i);
	}
	return -1;
}

void dgDockHolder::sort_y() {
	DockableGroup** a = array_;
	int count = children_ - 1;
	for(int i = 0; i < count; i++) {
		for(int j = 0; j < count - i; j++) {
			DockableGroup* a1 = *(a + j);
			DockableGroup* a2 = *(a + j + 1);
			DockableGroup* k;
			if(ABS(a1->y() - a2->y()) <= TOLERANCE) {
				if(a1->x() > a2->x()) {
					k = a[j + 1];
					a[j + 1] = a[j];
					a[j] = k;
				}
			}
			else {
				if(a1->y() > a2->y()) {
					k = a[j + 1];
					a[j + 1] = a[j];
					a[j] = k;
				}
			}
		}
	}
}

void dgDockHolder::sort_x() {
	DockableGroup** a = array_;
	int count = children_ - 1;
	for(int i = 0; i < count; i++) {
		for(int j = 0; j < count - i; j++) {
			DockableGroup* a1 = *(a + j);
			DockableGroup* a2 = *(a + j + 1);
			DockableGroup* k;
			if(ABS(a1->x() - a2->x()) <= TOLERANCE) {
				if(a1->y() > a2->y()) {
					k = a[j + 1];
					a[j + 1] = a[j];
					a[j] = k;
				}
			}
			else {
				if(a1->x() > a2->x()) {
					k = a[j + 1];
					a[j + 1] = a[j];
					a[j] = k;
				}
			}
		}
	}
}

void DockRegion::layout_dockh(int ix_) {
	dgDockHolder* holder_ = DockRegion::holder_[ix_];
		
	int tx = holder_->x();
	int ty = holder_->y();
	int tw = holder_->x() + holder_->w();
	
	int maxh = 0;
	int last_ = holder_->array_? (*(holder_->array_))->y() : 0;
	
	for(int i = 0; i < holder_->children_; i++) {
		DockableGroup *o = *(holder_->array_ + i);
		
		
		if(o->w() > tw) o->resize(tw, o->h());
		if((tx + o->w() > holder_->x() + tw) || (o->y() - last_ > TOLERANCE)) {
			ty += maxh;
			tx = holder_->x();
			maxh = 0;
		}
		maxh = (o->h() > maxh)? o->h() : maxh;
		
		if(o->fillable()) {
			o->resize(tw - tx, o->h());
		}
		last_ = o->y();
		if(ix_ == TOP_HOLDER) o->position(tx, ty);
		else o->set(tx, ty, o->w(), o->h());
		
		tx += o->w();
	}
	ty += maxh;
	
	if(ix_ == TOP_HOLDER) {
		holder_->set(holder_->x(), holder_->y(), holder_->w(), ty);
		return;
	}
	DockableGroup*const* a = holder_->array_;
	int ext = ty - h();
	for (int i = holder_->children_; i--;) {
		DockableGroup* o = *a++;
		o->position(o->x(), o->y() - ext);
	}
	holder_->set(holder_->x(), holder_->y() - ext, holder_->w(), holder_->h() + ext);
}

void DockRegion::layout_dockv(int ix_) {
	dgDockHolder* holder_ = DockRegion::holder_[ix_];
		
	int tx = holder_->x();
	int ty = holder_->y();
	int th = holder_->y() + holder_->h();
	
	int maxw = 0;
	int last_ = holder_->array_? (*(holder_->array_))->x() : 0;
	
	for(int i = 0; i < holder_->children_; i++) {
		DockableGroup *o = *(holder_->array_ + i);
		
		if(o->h() > th) o->resize(o->w(), th);
		if((ty + o->h() > holder_->y() + th) || (o->x() - last_ > TOLERANCE)) {
			tx += maxw;
			ty = holder_->y();
			maxw = 0;
		}
		maxw = (o->w() > maxw)? o->w() : maxw;
		
		if(o->fillable()) {
			o->resize(o->w(), th - ty);
		}
		last_ = o->x();
		if(ix_ == LEFT_HOLDER) o->position(tx, ty);
		else o->set(tx, ty, o->w(), o->h());
		
		ty += o->h();
	}
	tx += maxw;
	
	if(ix_ == LEFT_HOLDER) {
		holder_->set(holder_->x(), holder_->y(), tx, holder_->h());
		return;
	}
	DockableGroup*const* a = holder_->array_;
	int ext = tx - w();
	for (int i = holder_->children_; i--;) {
		DockableGroup* o = *a++;
		o->position(o->x() - ext, o->y());
	}
	holder_->set(holder_->x() - ext, holder_->y(), holder_->w() + ext, holder_->h());
}

void DockRegion::layout_holder() {
	int preX, preY, preW, preH;
	
	preY = holder_[TOP_HOLDER]->y();
	preW = holder_[TOP_HOLDER]->w();
	holder_[TOP_HOLDER]->set(0, 
													 0, 
													 w(), 
													 holder_[TOP_HOLDER]->h());
	if(preY != holder_[TOP_HOLDER]->y() || preW != holder_[TOP_HOLDER]->w()) 
		layout_dockh(TOP_HOLDER);
	holder_[TOP_HOLDER]->resize_range();
	
	preY = holder_[BOTTOM_HOLDER]->y();
	preW = holder_[BOTTOM_HOLDER]->w();
	holder_[BOTTOM_HOLDER]->set(0, 
															h() - holder_[BOTTOM_HOLDER]->h(), 
															w(), 
															holder_[BOTTOM_HOLDER]->h());
	if(preY != holder_[BOTTOM_HOLDER]->y() || preW != holder_[BOTTOM_HOLDER]->w()) 
		layout_dockh(BOTTOM_HOLDER);
	holder_[BOTTOM_HOLDER]->resize_range();
	
	preY = holder_[LEFT_HOLDER]->y();
	preH = holder_[LEFT_HOLDER]->h();
	holder_[LEFT_HOLDER]->set(0, 
														holder_[TOP_HOLDER]->h(), 
														holder_[LEFT_HOLDER]->w(), 
														h() - holder_[TOP_HOLDER]->h() - holder_[BOTTOM_HOLDER]->h());
	if(preY != holder_[LEFT_HOLDER]->y() || preH != holder_[LEFT_HOLDER]->h()) 
		layout_dockv(LEFT_HOLDER);
	holder_[LEFT_HOLDER]->resize_range();
	
	preY = holder_[RIGHT_HOLDER]->y();
	preH = holder_[RIGHT_HOLDER]->h();
	holder_[RIGHT_HOLDER]->set(w() - holder_[RIGHT_HOLDER]->w(), 
														holder_[TOP_HOLDER]->h(), 
														holder_[RIGHT_HOLDER]->w(), 
														h() - holder_[TOP_HOLDER]->h() - holder_[BOTTOM_HOLDER]->h());
	if(preY != holder_[RIGHT_HOLDER]->y() || preH != holder_[RIGHT_HOLDER]->h()) 
		layout_dockv(RIGHT_HOLDER);
	holder_[RIGHT_HOLDER]->resize_range();
	
	widget_->resize(holder_[LEFT_HOLDER]->w(),
									holder_[TOP_HOLDER]->h(),
									w() - holder_[RIGHT_HOLDER]->w() - holder_[LEFT_HOLDER]->w(),
									h() - holder_[BOTTOM_HOLDER]->h() - holder_[TOP_HOLDER]->h());
}

void DockRegion::layout() {
	layout_holder();
}

void DockRegion::screenxy(int &x_, int &y_) {
	Widget *cw = this;
	x_ = y_ = 0;
	do {
		x_ += cw->x();
		y_ += cw->y();
	} while(cw = cw->parent());
}

DockRegion::DockRegion(int X, int Y, int W, int H,const char *l)
	: Group(X, Y, W, H, l) {
	box(FLAT_BOX);
	
	holder_[TOP_HOLDER] = new dgDockHolder(0, 0, w(), 0, ALIGN_TOP);
	holder_[BOTTOM_HOLDER] = new dgDockHolder(0, h(), w(), 0, ALIGN_BOTTOM);
	holder_[LEFT_HOLDER] = new dgDockHolder(0, 0, 0, h(), ALIGN_LEFT);
	holder_[RIGHT_HOLDER] = new dgDockHolder(w(), 0, 0, h(), ALIGN_RIGHT);
}

DockRegion::~DockRegion() {}


int DockRegion::get_holder(DockableGroup *d) {
	for(int i = 0; i < HOLDER_NO; i++) {
		if(holder_[i]->have(d) != -1) return i;
	}
	return -1;
}

void DockRegion::draw() {
	Group::draw();

// 	for(int i = 0; i < HOLDER_NO; i++) {
// 		setcolor(RED);
// 		strokerect(Rectangle(
// 						holder_[i]->x(), 
// 						holder_[i]->y(), 
// 						holder_[i]->w(), 
// 						holder_[i]->h()));
// 		setcolor(GREEN);
// 		Rectangle r = holder_[i]->range();
// 		strokerect(r);
// 	}
}

int fltk::DockRegion::check_dock(DockableGroup *d) {	
	int x_, y_;
	screenxy(x_, y_);
	int lx = d->window()->x() - x_;
	int ly = d->window()->y() - y_;
	
	for(int i = 0; i < HOLDER_NO; i++) {
		if(holder_[i]->range_.contains(lx, ly)) {			
			holder_[i]->add(d);
			add(d);
			int x_, y_;
			screenxy(x_, y_);
			d->position(d->window()->x() - x_, d->window()->y() - y_);
			d->window()->hide();
				
			switch(i) {
				case TOP_HOLDER:
					d->orientation(DockableGroup::ORIENTED_HORIZONTAL);
					holder_[i]->sort_y();
					layout_dockh(i);
					layout_holder();
					return DockableGroup::DOCKED_TOP;
				case BOTTOM_HOLDER:
					d->orientation(DockableGroup::ORIENTED_HORIZONTAL);
					holder_[i]->sort_y();
					layout_dockh(i);
					layout_holder();
					return DockableGroup::DOCKED_BOTTOM;
				case LEFT_HOLDER:
					d->orientation(DockableGroup::ORIENTED_VERTICAL);
					holder_[i]->sort_x();
					layout_dockv(i);
					layout_holder();
					return DockableGroup::DOCKED_LEFT;
				case RIGHT_HOLDER:
					d->orientation(DockableGroup::ORIENTED_VERTICAL);
					holder_[i]->sort_x();
					layout_dockv(i);
					layout_holder();
					return DockableGroup::DOCKED_RIGHT;
			}
			break;
		}
	}
	return DockableGroup::DOCKED_NONE;
}

void DockRegion::docked_layout(uchar a) {
	switch(a) {
		case TOP_HOLDER:
		case BOTTOM_HOLDER:
			layout_dockh(a);
			break;
		case LEFT_HOLDER:
		case RIGHT_HOLDER:
			layout_dockv(a);
	}
	layout_holder();
}

int DockRegion::dock(DockableGroup *d, uchar a) {
	d->window()->hide();
	holder_[a]->add(d);
	add(d);
	switch(a) {
		case TOP_HOLDER:
			d->orientation(DockableGroup::ORIENTED_HORIZONTAL);
			d->position(0, holder_[a]->y() + holder_[a]->h());
			holder_[a]->sort_y();
			layout_dockh(a);
			layout_holder();
			return a;
		case BOTTOM_HOLDER:
			d->orientation(DockableGroup::ORIENTED_HORIZONTAL);
			d->position(0, holder_[a]->y());
			holder_[a]->sort_y();
			layout_dockh(a);
			layout_holder();
			return a;
		case LEFT_HOLDER:
			d->orientation(DockableGroup::ORIENTED_VERTICAL);
			d->position(holder_[a]->x() + holder_[a]->w(), 0);
			holder_[a]->sort_x();
			layout_dockv(a);
			layout_holder();
			return a;
		case RIGHT_HOLDER:
			d->orientation(DockableGroup::ORIENTED_VERTICAL);
			d->position(holder_[a]->x(), 0);
			holder_[a]->sort_x();
			layout_dockv(a);
			layout_holder();
			return a;
	}
	return DockableGroup::DOCKED_NONE;
}

int DockRegion::check_undock(DockableGroup *d, uchar a) {
	switch(a) {
		case TOP_HOLDER:
		case BOTTOM_HOLDER:
			holder_[a]->sort_y();
		case LEFT_HOLDER:
		case RIGHT_HOLDER:
			holder_[a]->sort_x();
	}
	if(!holder_[a]->range_.contains(d->x(), d->y()))
		return undock(d, a);
	return a;
}

int DockRegion::undock(DockableGroup *d, uchar a) {
	remove(d);
	holder_[a]->remove(d);
	
	switch(a) {
		case TOP_HOLDER:
			holder_[TOP_HOLDER]->sort_y();
			layout_dockh(a);
		case BOTTOM_HOLDER:
			holder_[BOTTOM_HOLDER]->sort_y();
			layout_dockh(a);
		case LEFT_HOLDER:
			holder_[LEFT_HOLDER]->sort_x();
			layout_dockv(a);
		case RIGHT_HOLDER:
			holder_[RIGHT_HOLDER]->sort_x();
			layout_dockv(a);
	}
	layout_holder();
	d->showWindow();
	return DockableGroup::DOCKED_NONE;
}
	
//
// End of "$Id DockRegion.cxx Thu Feb 5 2006 z_hossain $".
//
