//
// "$Id DockRegion.h Thu Feb 5 2006 z_hossain $"
//
// Dockability for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2005 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to: 
//
//             md_z_hossain@yahoo.com
//

#ifndef fltk_DockRegion_h
#define fltk_DockRegion_h

#include <fltk/Window.h>
#include "DockableGroup.h"

#define HOLDER_NO 4

#define TOP_HOLDER 0
#define BOTTOM_HOLDER 1
#define LEFT_HOLDER 2
#define RIGHT_HOLDER 3

namespace fltk {

class dgDockHolder : public Rectangle {
	friend class DockRegion;
	
	int children_;
	DockableGroup **array_;
	Flags align_;
	Rectangle range_;
	
protected:
	void resize_range();
	
public:
	dgDockHolder(int, int, int, int, Flags);
	~dgDockHolder();

	void add(DockableGroup*);
	void remove(DockableGroup*);
	int have(DockableGroup*);
	void sort_x();
	void sort_y();
	void align_holder(Flags a) { align_ = a; resize_range(); }
	
	void range(int x, int y, int w, int h) { range_.set(x, y, w, h); }
	Rectangle &range() { return range_; }
};

class DockRegion : public Group {
	Widget *widget_;
	dgDockHolder *holder_[HOLDER_NO];
		
	void screenxy(int&, int&);
	int get_holder(DockableGroup *);
	public:
		DockRegion( int, int, int, int, const char* = 0 );
		~DockRegion();
		void layout_dockh(int);
		void layout_dockv(int);
		void draw();
		void layout();
		void layout_holder();
		int check_dock(DockableGroup *);
		int dock(DockableGroup *, uchar);
		void docked_layout(uchar);
		int check_undock(DockableGroup *, uchar);
		int undock(DockableGroup *, uchar);
		Widget *widget() const { return widget_; }
		void widget(Widget *w) { add(w); widget_ = w; }
};

}
#endif

//
// End of "$Id DockRegion.h Thu Feb 5 2006 z_hossain $".
//
