//
// "$Id DockableGroup.cxx Thu Feb 5 2006 z_hossain $"
//
// Dockability for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2005 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to: 
//
//             md_z_hossain@yahoo.com
//

#include <string.h>
#include "DockableGroup.h"
#include "DockRegion.h"

using namespace fltk;

DockableGroup::DockableGroup(int X, int Y, int W, int H, const char *l)
	: Group(X, Y, W, H, l) {
	region_ = 0;
	regionno_ = 0;
	lsize_ = 0;
	able_ = DOCKABLE_ALL | ORIENTABLE_BOTH;
	state_ = DOCKED_NONE | ORIENTED_HORIZONTAL;
	
	window_ = new Window(X, Y, W, H, l);
	//window_->clear_border();
	window_->add(this);
	Group::current(this);
}

DockableGroup::~DockableGroup() {
	delete window_;
}

bool DockableGroup::resize(int x, int y, int w, int h) {
	bool b = Widget::resize(x, y, w, h);
	layout();
	return b;
}

void DockableGroup::add_region(DockRegion *r) {
	if (regionno_ == 0) {
		region_ = new DockRegion*[1];
		region_[0] = r;
	} else {
		if (!(regionno_ & (regionno_-1))) {
			DockRegion** newregion_ = new DockRegion*[2*regionno_];
			memcpy(newregion_, region_, regionno_*sizeof(DockRegion*));
			delete[] region_;
			region_ = newregion_;
		}
		region_[regionno_] = r;
	}
	++regionno_;
}

void DockableGroup::change_orientation(uchar a) {
	if(orientation() == a) return;
	state_ = (state_ & (~ORIENTED_MASK)) | a;
	dgSize *temp = new dgSize(w(), h());
	if(lsize_) resize(lsize_->w(), lsize_->h());
	else resize(h(), w());
	window_->resize(w(), h());
	delete lsize_;
	lsize_ = temp;
}

void DockableGroup::showWindow() {
	int x_ = window_->x();
	int y_ = window_->y();
	//if(window_) delete window_;
	window_ = new Window(x_, y_, w(), h());
	window_->clear_border();
	window_->add(this);
	position(0, 0);
	//this->resize(0, 0, window_->w(), window_->h());
	//this->show();
	window_->show();
	//window_->resize(w(), h());
}


//
// End of "$Id DockableGroup.cxx Thu Feb 5 2006 z_hossain $".
//
