//
// "$Id DockableGroup.h Thu Feb 5 2006 z_hossain $"
//
// Dockability for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2005 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to: 
//
//             md_z_hossain@yahoo.com
//


#ifndef fltk_DockableGroup_h
#define fltk_DockableGroup_h

#include <fltk/Window.h>

namespace fltk {

class DockRegion;

struct dgSize {
	int w_, h_;

	dgSize(int W, int H) : w_(W), h_(H) {}
	int w() { return w_; }
	int h() { return h_; }
};

class DockableGroup : public Group {
	
	DockRegion **region_;
	int regionno_;
	
	Window *window_;
	uchar state_;
	uchar able_;
	int min_w_, min_h_;
	dgSize *lsize_;
	
protected:
	virtual void change_orientation(uchar);
	
public:
	
	enum {
		DOCKED_TOP = 0x00,
		DOCKED_BOTTOM = 0x01,
		DOCKED_LEFT = 0x02,
		DOCKED_RIGHT = 0x03,
		DOCKED_NONE = 0x04,
		DOCKED_MASK = 0x07,
	
		ORIENTED_VERTICAL = 0x00,
		ORIENTED_HORIZONTAL = 0x8,
		ORIENTED_MASK = 0x8,
	};
	enum{
		DOCKABLE_TOP = 0x01,
		DOCKABLE_LEFT = 0x02,
		DOCKABLE_BOTTOM = 0x04,
		DOCKABLE_RIGHT = 0x08,
		DOCKABLE_NONE = 0x00,
		DOCKABLE_ALL = 0x0F,
		DOCKABLE_MASK = 0x0F,
	
		ORIENTABLE_VERTICAL = 0x10,
		ORIENTABLE_HORIZONTAL = 0x20,
		ORIENTABLE_BOTH = 0x30,
		ORIENTABLE_MASK = 0x30,
		
		FILLABLE = 0x40,
		FILLABLE_MASK = 0x40,
	};
	
	DockableGroup(int, int, int, int, const char* = 0);
	~DockableGroup();
	
	bool resize(int x, int y, int w, int h);
	bool resize(int w, int h) { return resize(x(), y(), w, h); }
	
	void insert(Widget &o, int index);
	void dockable(uchar a) { able_ = (able_ & (~DOCKABLE_MASK)) | a;}
	uchar dockable() const { return able_ & DOCKABLE_MASK; }

	void state(uchar a) { state_ = (state_ & (~DOCKED_MASK)) | a; }
	uchar state() const { return state_ & DOCKED_MASK; }

	void orientation(uchar a) { change_orientation(a); }
	uchar orientation() const { return state_ & ORIENTED_MASK; }

	void orientatable(uchar a) { able_ = (able_ & (~ORIENTABLE_MASK)) | a; }
	uchar orientatable() { return able_ & ORIENTABLE_MASK; }
	
	uchar fillable() { return able_ & FILLABLE_MASK; }
	void fillable(uchar a) { able_ =  (able_ & (~FILLABLE_MASK)) | a; }
	
	void add_region(DockRegion *);
	DockRegion *region(int i) const { return region_[i]; }
	
	int min_w()	{ return min_w_; }
	void min_w(int w) { min_w_ = w; }
	int min_h() { return min_h_; }
	void min_h(int h) { min_h_ = h; }
	
	void begin() { Group::current(this); }
	void end() { Group::current(0); }
	Window *window() { return window_; }
	void showWindow();
};

}
#endif

//
// End of "$Id DockableGroup.h Thu Feb 5 2006 z_hossain $".
//
