//
// "$Id ToolGroup.h Thu Feb 7 2006 z_hossain $"
//
// Dockability for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2005 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to: 
//
//             md_z_hossain@yahoo.com
//

#ifndef fltk_ToolGroup_h
#define fltk_ToolGroup_h

#include <fltk/PopupMenu.h>

#include "DockableGroup.h"

#define GRIPPER_GAP					5
#define GRIPPER_THICKNESS		8
#define INTERNAL_CHILD_NO		3
#define DIVIDER_GAP					7
#define DIVIDER_SHORTENED		5
#define DEFAULT_CHILD_INFO	0

namespace fltk {

class tgGripper : public PopupMenu {
	int thick_;
	static void menu_cb(Widget*, void*);
	static void float_cb(Widget*, void*);
	
	protected:
		void draw();
		int handle(int);
	public:
		tgGripper(int = GRIPPER_THICKNESS);
		int thickness() { return thick_; }
		void thickness(int t) { thick_ = t; }
};

class tgChildInfo {
	friend class ToolGroup;
	
	private:
		Widget *widget_;
		int iw_;
		int ih_;
		uchar flag_;
		
		
 	public:
		enum {
			PLAIN = 0,
			FIXED_WIDTH = 1,
			FIXED_HEIGHT = 2,
			TOOLBAR_DIVIDER = 4,
			INTERNAL_CHILD = 8,
		};
		tgChildInfo(Widget *, uchar = PLAIN);
		int iw() { return iw_; }
		int ih() { return ih_; }
};


class ToolGroup : public DockableGroup {
	tgGripper *gripper_;
	
	tgChildInfo **cinfo_;
	int lcinfo_;
	void insert(tgChildInfo *, int);
	void add(tgChildInfo *o) { insert(o, lcinfo_); }
	
public:
	enum {
		PLAIN = 0,
		FIXED_WIDTH = 1,
		FIXED_HEIGHT = 2,
		TOOLBAR_DIVIDER = 4,
		INTERNAL_CHILD = 8,
	};
	ToolGroup(int, int, int, int, const char * = 0);
	~ToolGroup();
	void layout();
	void init_child();
	void update_all();
	int handle(int);
	
	void child_info(int);
	void child_info(Widget *, int);
};

}
#endif

//
// End of "$Id ToolGroup.h Thu Feb 7 2006 z_hossain $".
//
